% function displayTemplateAndMatching

% Display active basis and adaboost templates;
% and display matched templates on example images.

sx = 85; sy = 127;

% Change the following parameter to control how many features to show for the active basis template. 
nF = 30; % show first 30 Gabor elements

%% active basis template
iRun = 12;
load(sprintf('sigmoidAdaptive/model_run%d.mat',iRun));

scale = .7;  % scale of Gabors
norient = 15;  % number of orientations
addpath('sigmoidAdaptive');
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 


Mx = Mx(1:nF); My = My(1:nF); Mi = Mi(1:nF); Mm = Mm(1:nF); Mm1 = Mm1(1:nF); Totalsketch = nF;
sym = zeros(sx,sy);
for k = 1:nF
    ori = Mi(k) + 1;
    col = My(k);
    row = Mx(k);
    sym = drawGaborSymbol( sym, allsymbol, row, col, ori, sqrt(Mm(k)) );
end
towrite = -sym;
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));

imwrite(towrite,sprintf('ActiveBasisTemplate_sketch%d.png',nF));
figure;imshow(towrite,[]);
print('-depsc',sprintf('ActiveBasisTemplate_sketch%d.eps',nF) );
close

%% Display Adaboost Max Template

load(sprintf('adaboostMax/out_adaboostMax_run%d.mat',iRun));

% <lattice> is the image locations to sub-sample from.
% Calculate <lattice>:
subsampleStep = 2;
h = 8; % half size of gabor filter
rowNums = (h+1):subsampleStep:sx-(h+1);
colNums = (h+1):subsampleStep:sy-(h+1);
indResponseMatrix = zeros(size(rowNums,2),size(colNums,2),3);
indResponseMatrix(:,:,1) = repmat(rowNums',1,size(colNums,2));
indResponseMatrix(:,:,2) = repmat(colNums,size(rowNums,2),1);
indResponseMatrix(:,:,3) = (indResponseMatrix(:,:,2) - 1) * sx + indResponseMatrix(:,:,1);
lattice = indResponseMatrix(:,:,3);
addpath('adaboostMax');
% lambdas = lambdas(1:nF); selectedFeatures = selectedFeatures(1:nF);
towrite = displaySelectedFilters(lambdas,selectedFeatures,lattice,[sx sy]);

imwrite(towrite,'AdaboostMaxTemplate.png');
figure;imshow(towrite,[]);
print('-depsc',sprintf('AdaboostMaxTemplate.eps') );
close

%% active basis template matching
load(sprintf('RandomSplits/split%d.mat',iRun),'posTestImages');
addpath('sigmoidAdaptive');
posFolder = 'posAll/';  % folder of positive testing images
posTestImagePath = cell(1,length(posTestImages));
for i = 1:length(posTestImages)
    posTestImagePath{i} = [posFolder posTestImages{i}];
end
% posTestImagePath = posTestImagePath(1:10);
nimage = length(posTestImagePath); % # images
Isize = zeros(nimage, 2);
I = cell(1,nimage);
for i = 1 : nimage
    tmpIm = imread(posTestImagePath{i});
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = double(tmpIm); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the testing images the same

Sx = zeros(1, nimage) + sx; 
Sy = zeros(1, nimage) + sy; 

% filter the testing images 
disp('start filtering');
tic
allfiltered = applyfilterfftsame(I, allfilter);  % filter testing images
disp(['filtering time: ' num2str(toc) ' seconds']);

% sigmoid transformation
Ctransform(nimage, norient, allfiltered, Sx, Sy, Upperbound);

% prepare the spaces for C code
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for i = 1 : nimage
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, Totalsketch);  % total scores for testing images

% pass on the pointers to C for heavy computation
disp('start testing');
tic
bkp = zeros([nimage norient]);
Ctesting(nimage, norient, allfiltered, C, h, sx, sy,...
    Mi, Mx, My, Mm, Mm1, allsymbol(1, :), Asym, Lrange,...
    Orange, sub, Totalsketch, Upperbound, tss, M, lam, e, lz, bkp);

K = nimage;
[tsssort, ind] = sort(tss);
J = cell(1,nimage*2+2);
J{1} = -sym;
J{2} = [];
for i = 1 : K
     
    J{2*i+1} = I{ind(nimage-i+1)};
    J{2*i+2} = -(Asym{ind(nimage-i+1)}); 
end
% J = J([1:2 11:20]);
towrite = displayImages(J,12,sx,sy);
imwrite(towrite,'matched.png');
figure;imshow(towrite,[]);
print('-depsc',sprintf('matched.eps') );
close

%% Display the first two principal components
load(sprintf('pca/pca_result_run%d.mat',iRun));
towrite = reshape(meanIm,[sx sy]);
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
imwrite(towrite,'meanIm.png');
figure;imshow(towrite,[]);
print('-depsc',sprintf('meanIm.eps') );
close
towrite = reshape(PC(1,:),[sx sy]);
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
imwrite(towrite,'pc1.png');
figure;imshow(towrite,[]);
print('-depsc',sprintf('pc1.eps') );
close
towrite = reshape(PC(2,:),[sx sy]);
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
imwrite(towrite,'pc2.png');
h=figure;imshow(towrite,[]);
print('-depsc',sprintf('pc2.eps') );
close

