% function eigenface
clear
% Eigenface for classification with cross validation.

% Constant
numComponents = 20;
% height = 85; width = 127; % original image size
sx = 85; sy = 127; % resized image size

addpath(['../']);

for iRun = 1:25
    disp(['run ' num2str(iRun)]);
    load(sprintf('../RandomSplits/split%d',iRun),'posTrainImages');
    % load training positives
    dataMatrix = zeros(length(posTrainImages),sx*sy);
    eigenfaces = zeros(numComponents,sx*sy);
    for i = 1:length(posTrainImages)
        im = imread(['../posAll/' posTrainImages{i}]);
        if size(im,3) == 3
            im = rgb2gray(im);
        end
        im = double(imresize(im,[sx sy]));
        im = im - mean(im(:));
        im = im / sqrt(var(im(:)));
        dataMatrix(i,:) = reshape(im,1,sx*sy);
    end
    % substracting the mean image
    meanIm = mean(dataMatrix, 1);
    dataMatrix = dataMatrix - repmat(meanIm, size(dataMatrix,1), 1);
    % perform PCA
    [COEFF, SCORE, latent] = princomp(dataMatrix);
    eigenfaces(1:numComponents,:) = COEFF(:,1:numComponents)';
    
    % testing
    load(sprintf('../RandomSplits/split%d',iRun),'posTestImages','negTestImages');
    
    scorePos = zeros(length(posTestImages),numComponents);
    for i = 1:length(posTestImages)
        im = imread(['../posAll/' posTestImages{i}]);
        if size(im,3) == 3
            im = rgb2gray(im);
        end
        im = double(imresize(im,[sx sy]));
        im = im - mean(im(:));
        im = im / sqrt(var(im(:)));
        im = reshape(im,1,sx*sy);
        % substract mean
        im = im - meanIm;
        % compute the L2 distance
        for j = 1:numComponents
            im = im - sum(im.*eigenfaces(j,:)) * eigenfaces(j,:);
            score = exp(-sum(im.^2)/sx/sy);
            scorePos(i,j) = score;
        end
        
    end
    
    scoreNeg = zeros(length(negTestImages),numComponents);
    for i = 1:length(negTestImages)
        im = imread(['../negAll/' negTestImages{i}]);
        if size(im,3) == 3
            im = rgb2gray(im);
        end
        im = double(imresize(im,[sx sy]));
        im = im - mean(im(:));
        im = im / sqrt(var(im(:)));
        im = reshape(im,1,sx*sy);
        % substract mean
        im = im - meanIm;
        % compute the L2 distance
        for j = 1:numComponents
            im = im - sum(im.*eigenfaces(j,:)) * eigenfaces(j,:);
            score = exp(-sum(im.^2)/sx/sy);
            scoreNeg(i,j) = score;
        end

    end
%     allScore = [scorePos;scoreNeg]; % scores of all test images
%     gt = [ones(size(scorePos,1),1);-1*ones(size(scoreNeg,1),1)]; % ground truth of all test images
%     testAUC = calc_auc(allScore,gt); % obtain ROC curve by comparing predicted scores and ground truth
%     [tp, fp] = roc(gt,allScore); % true positive and false postive rate
%     ap = getAP(gt,allScore);
    [ap testAUC tp fp] = evaluate(scorePos,scoreNeg,1);
    save(sprintf('eigenface_result_run%d.mat',iRun),...
        'scoreNeg','scorePos','eigenfaces','tp','fp','testAUC','ap');
end